from paho.mqtt import client as mqtt_client
import json
import logging
import random
import time
from Control_Interface.controlInterface import controlInterface



BROKER = 'broker.hivemq.com' 
PORT = 1883
TOPIC_SUB = "python-mqtt-sub"
TOPIC_PUB = "python-mqtt-pub"
CLIENT_ID = f'python-mqtt-{random.randint(0, 1000)}'


FIRST_RECONNECT_DELAY = 1 
RECONNECT_RATE = 2
MAX_RECONNECT_COUNT = 12
MAX_RECONNECT_DELAY = 60

FLAG_EXIT = False

interface = controlInterface()

print(interface.getListIp())
interface.newNhr("9410")
interface.newNhr("9430")

NHRs = {  
     "9410" : interface.getNhr9410()[0],
     "9430" : interface.getNhr9430()[0], 
}

def on_connect(client, userdata, flags, rc):
    if rc == 0 and client.is_connected():
        print("Connected to MQTT Broker!")
        client.subscribe(TOPIC_SUB)
    else:
        print(f'Failed to connect, return code {rc}')


def on_disconnect(client, userdata, rc):
    logging.info("Disconnected with result code: %s", rc)
    reconnect_count, reconnect_delay = 0, FIRST_RECONNECT_DELAY
    while reconnect_count < MAX_RECONNECT_COUNT:
        logging.info("Reconnecting in %d seconds...", reconnect_delay)
        time.sleep(reconnect_delay)

        try:
            client.reconnect()
            logging.info("Reconnected successfully!")
            return
        except Exception as err:
            logging.error("%s. Reconnect failed. Retrying...", err)

        reconnect_delay *= RECONNECT_RATE
        reconnect_delay = min(reconnect_delay, MAX_RECONNECT_DELAY)
        reconnect_count += 1
    logging.info("Reconnect failed after %s attempts. Exiting...", reconnect_count)
    global FLAG_EXIT
    FLAG_EXIT = True


def on_message(client, userdata, message):
    print(f'Received `{message.payload.decode()}` from `{message.topic}` topic')
    message = json.loads(message.payload.decode())
    
    selector = message['selector']
    fn = message['fn']
    args = message.get('args', [])

    print(message['selector'])
    nhr = NHRs[selector]

    result = getattr(nhr, fn)(*args)

    client.publish(TOPIC_PUB, json.dumps(result))

def connect_mqtt():
    client = mqtt_client.Client(CLIENT_ID)
    client.on_connect = on_connect
    client.on_message = on_message
    client.connect(BROKER, PORT, keepalive=3)
    client.on_disconnect = on_disconnect
    return client


def publish(client):
    msg_count = 0
    while not FLAG_EXIT:
        msg_dict = {
            'msg': msg_count
        }
        msg = json.dumps(msg_dict)
        if not client.is_connected():
            logging.error("publish: MQTT client is not connected!")
            time.sleep(1)
            continue
        result = client.publish(TOPIC_PUB, msg)
        # result: [0, 1]
        status = result[0]
        if status == 0:
            print(f'Send `{msg}` to topic `{TOPIC_SUB}`')
        else:
            print(f'Failed to send message to topic {TOPIC_SUB}')
        msg_count += 1
        time.sleep(1)


def run():
    logging.basicConfig(format='%(asctime)s - %(levelname)s: %(message)s',
                        level=logging.DEBUG)
    client = connect_mqtt()
    client.loop_start()
    time.sleep(1000)
    if client.is_connected():
        publish(client)
    else:
        client.loop_stop()


if __name__ == '__main__':
    run()
